#include <RootWidget.hxx>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qdatetime.h>
#include <qlineedit.h>
#include <qtextview.h>
#include <qfile.h>
#include <qpainter.h>
#include <qapplication.h>

#include <TROOT.h>
#include "TEnv.h"
#include "TGuiFactory.h"
#include <TCanvas.h>
#include <TVirtualX.h>
#include <TSystem.h>
#include <TApplication.h>
#include <TPluginManager.h>
#include <TBrowser.h>
#include <TQApplication.h>
#include <TFormula.h>
#include <TF1.h>
#include <TH1.h>
#include <TH3F.h>
#include <TFrame.h>
#include <TStyle.h>
#include <TGeoManager.h>

//--------------------------------------------------------------------------------

EWO_PLUGIN( RootWidget )

static TApplication *gPVSSRootApp = 0;

static QTimer *gRootTimer = 0;

TROOT app("PVSSRoot", "PVSS and Root integration");

//--------------------------------------------------------------------------------

QRootCanvas::QRootCanvas(QWidget *parent) : QWidget(parent), fCanvas(0)
{
   setMinimumSize(500, 400);
   setMouseTracking(kTRUE);
   setCursor( Qt::crossCursor );
}

void QRootCanvas::processRootEvents()
{
   if (qApp->tryLock()) {
      qApp->lock();
      gSystem->ProcessEvents();
      qApp->unlock();
   }
}

void QRootCanvas::closeEvent( QCloseEvent * )
{
   if (fCanvas) {
      delete fCanvas;
      fCanvas = 0;
   }
}

void QRootCanvas::mouseMoveEvent(QMouseEvent *e)
{
   if (fCanvas) {
      if (e->state() & LeftButton) {
         fCanvas->HandleInput(kButton1Motion, e->x(), e->y());
      } else {
         fCanvas->HandleInput(kMouseMotion, e->x(), e->y());
      }
   }
}


void QRootCanvas::mousePressEvent( QMouseEvent *e )
{
   if (fCanvas) {
      switch (e->button()) {
         case LeftButton :
            fCanvas->HandleInput(kButton1Down, e->x(), e->y());
            break;
         case MidButton :
            fCanvas->HandleInput(kButton2Down, e->x(), e->y());
            break;
         case RightButton :
            fCanvas->HandleInput(kButton3Down, e->x(), e->y());
            break;
         default:
            break;
      }
   }
}

void QRootCanvas::mouseReleaseEvent( QMouseEvent *e )
{
   if (fCanvas) {
      switch (e->button()) {
         case LeftButton :
            fCanvas->HandleInput(kButton1Up, e->x(), e->y());
            break;
         case MidButton :
            fCanvas->HandleInput(kButton2Up, e->x(), e->y());
            break;
         case RightButton :
            fCanvas->HandleInput(kButton3Up, e->x(), e->y());
            break;
         default:
            break;
      }
   }
}

void QRootCanvas::resizeEvent( QResizeEvent *e )
{
   QWidget::resizeEvent( e );
   if (fCanvas) {
      fCanvas->Resize();
      fCanvas->Update();
   }
}

void QRootCanvas::paintEvent( QPaintEvent *e )
{
   QWidget::paintEvent( e );
   if (fCanvas) {
      fCanvas->Resize();
      fCanvas->Update();
   }
   else {
      if (gApplication) {
//         gStyle->SetCanvasPreferGL(kTRUE);
         wid = gVirtualX->AddWindow((ULong_t)winId(),width(),height());
         fCanvas = new TCanvas("PVSS Root Canvas",width(),height(),wid);
//         fCanvas->SetSupportGL(kTRUE);
         fCanvas->Resize();
         fCanvas->Update();
      }
   }
}

void QRootCanvas::paintHisto( )
{
   if (fCanvas) {
      if (gApplication) {
         fCanvas->cd();
         if (gGeoManager && gGeoManager->GetVolume("CRYO")) {
            gGeoManager->GetVolume("CRYO")->Draw();
            fCanvas->Update();
         }
         else {
            fCanvas->Clear();
            TGeoManager::Import("http://root.cern.ch/files/atlas.root");
            gGeoManager->DefaultColors();
            gGeoManager->SetVisLevel(4);
            gGeoManager->GetVolume("CEE9")->InvisibleAll();
            gGeoManager->GetVolume("CEEh")->InvisibleAll();
            gGeoManager->GetVolume("CEE0")->SetTransparency(70);
            gGeoManager->GetVolume("CBE0")->SetTransparency(80);
            gGeoManager->GetVolume("CEEA")->SetTransparency(60);
            gGeoManager->GetVolume("CEEu")->SetTransparency(60);
            gGeoManager->GetVolume("CEEt")->SetTransparency(60);
            gGeoManager->GetVolume("CRE4")->SetTransparency(60);
            gGeoManager->GetVolume("CRE5")->SetTransparency(60);
            gGeoManager->GetVolume("CRE7")->SetTransparency(60);
            gGeoManager->GetVolume("CRYO")->Draw();
            fCanvas->Update();
         }
      }
   }
}

void QRootCanvas::openGLViewer( )
{
   if (fCanvas) {
      if (gApplication) {
         fCanvas->cd();
         gPad->GetViewer3D("ogl");
      }
   }
}

//--------------------------------------------------------------------------------

MyWidget::MyWidget(QWidget *parent) : QWidget(parent)
{
}

//--------------------------------------------------------------------------------

RootWidget::RootWidget(QWidget *parent)
  : BaseExternWidget(parent)
{
   baseWidget = new MyWidget(parent);

   QVBoxLayout *l = new QVBoxLayout(baseWidget);
   l->addWidget(canvas = new QRootCanvas(baseWidget));
   l->addWidget(b = new QPushButton("&Init Root", baseWidget));
   l->addWidget(v = new QPushButton("Open &GLViewer", baseWidget));
   l->addWidget(le = new QLineEdit(baseWidget));
   l->addWidget(tv = new QTextView(baseWidget));
   connect(b, SIGNAL(clicked()), this, SLOT(clicked1()));
   connect(v, SIGNAL(clicked()), this, SLOT(clicked2()));
   connect(le, SIGNAL(returnPressed()), this, SLOT(onCommandLine()));
   v->setDisabled(kTRUE);
}

//--------------------------------------------------------------------------------

QStringList RootWidget::signalList() const
{
  QStringList list;
  list.append("clicked1()");
  list.append("clicked2()");
  return list;
}

//--------------------------------------------------------------------------------

void RootWidget::clicked1()
{
   int argc = 1;
   char *argv[] = {
      "PVSSRoot",
      NULL
   };
   if (gApplication) {
      canvas->paintHisto();
      v->setEnabled(kTRUE);
   }
   else {
      new TApplication("PVSSRoot", &argc, argv);
      gApplication->SetReturnFromRun(kTRUE);
      b->setText("Draw &Geometry");
      canvas->repaint( true );
   }
   if (gRootTimer == 0) {
      gRootTimer = new QTimer( qApp );
   }
   if (!gRootTimer->isActive()) {
      QObject::connect( gRootTimer, SIGNAL(timeout()), canvas, SLOT(processRootEvents()) );
      gRootTimer->start( 20, FALSE );
   }
   emit signal("clicked1");
}

//--------------------------------------------------------------------------------

void RootWidget::clicked2()
{
   canvas->openGLViewer();
   emit signal("clicked2");
}

//--------------------------------------------------------------------------------

void RootWidget::onCommandLine()
{

   if (!gApplication) return;

   // get command string
   const char *cmd = le->text();
   char opt[2];
   // form temporary file path
   TString pathtmp = Form("%s/tempcmd", gSystem->TempDirectory());

   // if opt == "w" : open in write mode
   // if opt == "a" : open in append mode
   sprintf(opt, "w");

   gSystem->RedirectOutput(pathtmp.Data(), opt);

   // execute command line
   gApplication->ProcessLine(cmd);

   QFile file(pathtmp.Data());
   if (file.open(IO_ReadOnly)) {
      QTextStream stream( &file );
      tv->setText(stream.read());
      tv->scrollToBottom();
   }
   // restore back stdout/stderr
   gSystem->RedirectOutput(0);

   emit signal("onCommandLine");
}

//--------------------------------------------------------------------------------

QWidget *RootWidget::widget() const
{
  return baseWidget;
}

//--------------------------------------------------------------------------------
